/*
 *
 *  Copyright (C) 1996-99, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  dcmimgle
 *
 *  Author:  Joerg Riesmeier
 *
 *  Purpose: DicomInputPixel (Header)
 *
 *  Last Update:      $Author: joergr $
 *  Update Date:      $Date: 1999/03/24 17:20:02 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/dcmimgle/include/diinpx.h,v $
 *  CVS/RCS Revision: $Revision: 1.6 $
 *  Status:           $State: Exp $
 * 
 *   CVS/RCS Log at end of file
 *
 */


#ifndef __DIINPX_H
#define __DIINPX_H

#include "osconfig.h"
#include "dctypes.h"

#include "diutils.h"


/*---------------------*
 *  class declaration  *
 *---------------------*/

/** Abstract base class to convert DICOM pixel stream to intermediate representation
 */
class DiInputPixel 
{

 public:

    DiInputPixel(const unsigned int bits);

    virtual ~DiInputPixel();
    
    virtual int determineMinMax() = 0;
    
    virtual EP_Representation getRepresentation() const = 0;

    virtual void *getData() const = 0;

    virtual double getMinValue() const = 0;

    virtual double getMaxValue() const = 0;

    inline unsigned int getBits() const
    {
        return Bits;
    }
    
    inline double getAbsMinimum() const
    {
        return AbsMinimum;
    }
    
    inline double getAbsMaximum() const
    {
        return AbsMaximum;
    }
    
    inline double getAbsMaxRange() const
    {
        return AbsMaximum - AbsMinimum + 1;
    }
    
    inline unsigned long getCount() const
    {
        return Count;
    }
    

 protected:

    unsigned long Count;
    
    unsigned int Bits;
    double AbsMinimum;
    double AbsMaximum;
};


#endif


/*
 *
 * CVS/RCS Log:
 * $Log: diinpx.h,v $
 * Revision 1.6  1999/03/24 17:20:02  joergr
 * Added/Modified comments and formatting.
 *
 * Revision 1.5  1999/02/03 17:03:47  joergr
 * Added member variable and related methods to store number of bits used for
 * pixel data.
 *
 * Revision 1.4  1999/01/20 15:00:54  joergr
 * Added routine to calculate absolute range of pixel data.
 *
 * Revision 1.3  1998/12/22 14:18:40  joergr
 * Added implementation of methods to return member variables AbsMinimum/
 * Maximum.
 *
 * Revision 1.2  1998/12/16 16:30:34  joergr
 * Added methods to determine absolute minimum and maximum value for given
 * value representation.
 *
 * Revision 1.1  1998/11/27 15:06:38  joergr
 * Added copyright message.
 *
 * Revision 1.3  1998/05/11 14:53:17  joergr
 * Added CVS/RCS header to each file.
 *
 *
 */
