/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** Service Class Executive (SCE)
**
** Author: 	Andrew Hewett
**		Universitaet Oldenburg, Germany
** Created:	08/1993
**
** Last Update:		$Author: andreas $
** Update Date:		$Date: 1997/08/05 07:38:19 $
** Source File:		$Source: /share/dicom/cvs-depot/dcmtk/imagectn/apps/sce.cc,v $
** CVS/RCS Revision:	$Revision: 1.4 $
** Status:		$State: Exp $
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
END_EXTERN_C

#include "dicom.h"
#include "imagectn.h"
#include "dimse.h"
#include "diutil.h"
#include "sce.h"
#include "imagedb.h"
#include "cnf.h"
#ifndef NODISPLAY
#include "dispuser.h"
#endif

#include "sceecho.h"
#include "scestore.h"
#include "scefind.h"
#include "scemove.h"



/*
 *
 */

static CONDITION
SCE_dispatch(T_ASC_Association *assoc)
{
    CONDITION cond = DIMSE_NORMAL;
    CONDITION dbcond = DB_NORMAL;
    T_DIMSE_Message msg;
    T_ASC_PresentationContextID presID;
    DB_Handle *dbHandle = NULL;
    char *myAE = NULL;
    
    myAE = assoc->params->DULparams.calledAPTitle;
    
    /* Create a database handle for this association */
    dbcond = DB_createHandle(CNF_getStorageArea(myAE), 
	CNF_getMaxStudies(myAE), 
	CNF_getMaxBytesPerStudy(myAE), &dbHandle); 
    if (dbcond != DB_NORMAL) {
        errmsg("SCE_dispatch: cannot create DB Handle");
	cond = dbcond;	/* will abort association */
    }

    while (cond == DIMSE_NORMAL) {
	
	cond = DIMSE_receiveCommand(assoc, DIMSE_BLOCKING, 0, &presID,
		&msg, NULL);

	/* did peer release, abort, or do we have a valid message ? */
	switch (cond) {
	case DIMSE_NORMAL:
	    /* process command */
	    switch (msg.CommandField) {
	    case DIMSE_C_ECHO_RQ:
		cond = SCE_echoSCP(assoc, &msg.msg.CEchoRQ, presID);
		break;
	    case DIMSE_C_STORE_RQ:
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_StoreOperation,
			DSU_StartOperation, DSU_Incoming, 0);
#endif
		cond = SCE_storeSCP(assoc, &msg.msg.CStoreRQ, presID,
		    dbHandle);
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_StoreOperation,
			DSU_StopOperation, DSU_Incoming, 100);
#endif
		break;
	    case DIMSE_C_FIND_RQ:
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_FindOperation,
			DSU_StartOperation, DSU_Incoming, 0);
#endif
		cond = SCE_findSCP(assoc, &msg.msg.CFindRQ, presID, 
		    dbHandle);
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_FindOperation,
			DSU_StopOperation, DSU_Incoming, 100);
#endif
		break;
	    case DIMSE_C_MOVE_RQ:
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_MoveOperation,
			DSU_StartOperation, DSU_Incoming, 0);
#endif
		cond = SCE_moveSCP(assoc, &msg.msg.CMoveRQ, presID,
		    dbHandle);
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_MoveOperation,
			DSU_StopOperation, DSU_Incoming, 100);
#endif
		break;
	    case DIMSE_C_CANCEL_RQ:
		/* This is a late cancel request, just ignore it */
		if (verbose) {
		    printf("SCE_dispatch: late C-CANCEL-RQ, ignoring\n");
		}
		break;
	    default:
		/* we cannot handle this kind of message */
		cond = DIMSE_BADCOMMANDTYPE;
		errmsg("Cannot handle command: 0x%x\n", 
			(unsigned)msg.CommandField);
	        /* the condition will be returned, the caller
	         * will abort the assosiation.
	         */
	    }
	    break;
	case DIMSE_PEERREQUESTEDRELEASE:
	case DIMSE_PEERABORTEDASSOCIATION:
	    /* association gone */
	    break;
	default:
	    /* the condition will be returned, the caller
	     * will abort the assosiation.
	     */
	    break;
	}
    }

    /* Association done, destroy db handle */
    dbcond = DB_destroyHandle(&dbHandle);
    if (dbcond != DB_NORMAL) {
        errmsg("SCE_dispatch: cannot destroy DB Handle");
    }

    return cond;
}

CONDITION
SCE_handleAssociation(T_ASC_Association * assoc)
{
    CONDITION           cond;
    DIC_NODENAME	peerHostName;
    DIC_AE		peerAETitle;
    DIC_AE		myAETitle;

    ASC_getPresentationAddresses(assoc->params, peerHostName, NULL);
    ASC_getAPTitles(assoc->params, peerAETitle, myAETitle, NULL);

#ifndef NODISPLAY
    const char		*vendorName;
    /* inform display that an association has started */
    vendorName = CNF_vendorForPeerAETitle(peerAETitle);
    if (vendorName != NULL) {
        DSU_startAssociation(vendorName, peerAETitle, myAETitle,
	    DSU_Acceptor, DSU_MainAssociation);
    } else {
        DSU_startAssociation(peerHostName, peerAETitle, myAETitle,
	    DSU_Acceptor, DSU_MainAssociation);
    }
#endif
    
 /* now do the real work */
    cond = SCE_dispatch(assoc);

 /* clean up on association termination */
    if (cond == DIMSE_PEERREQUESTEDRELEASE) {
	COND_PopCondition(OFFalse);	/* pop only the peer requested
					 * release condition from the stack */
	if (verbose)
	    printf("Association Release\n");
	cond = ASC_acknowledgeRelease(assoc);
	ASC_dropSCPAssociation(assoc);
    } else if (cond == DIMSE_PEERABORTEDASSOCIATION) {
	COND_PopCondition(OFFalse);	/* pop DIMSE abort */
	COND_PopCondition(OFFalse);	/* pop DUL abort */
	if (verbose)
	    printf("Association Aborted\n");
    } else {
	errmsg("DIMSE Failure (aborting association):\n");
	COND_DumpConditions();
    /* some kind of error so abort the association */
	cond = ASC_abortAssociation(assoc);
    }

    cond = ASC_dropAssociation(assoc);
    if (!SUCCESS(cond)) {
	fprintf(stderr, "Cannot Drop Association:\n");
	COND_DumpConditions();
    }
    cond = ASC_destroyAssociation(&assoc);
    if (!SUCCESS(cond)) {
	fprintf(stderr, "Cannot Destroy Association:\n");
	COND_DumpConditions();
    }

#ifndef NODISPLAY
    /* inform display that the association has ended */
    DSU_stopAssociation(DSU_MainAssociation);
#endif

    return cond;
}


/*
** CVS Log
** $Log: sce.cc,v $
** Revision 1.4  1997/08/05 07:38:19  andreas
** Corrected error in DUL finite state machine
** SCPs shall close sockets after the SCU have closed the socket in
** a normal association release. Therfore, an ARTIM timer is described
** in DICOM part 8 that is not implemented correctly in the
** DUL. Since the whole DUL finite state machine is affected, we
** decided to solve the proble outside the fsm. Now it is necessary to call the
** ASC_DropSCPAssociation() after the calling ASC_acknowledgeRelease().
**
** Revision 1.3  1997/07/21 08:59:43  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.2  1996/09/27 08:46:20  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C.
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
