/*
 *
 *  Copyright (C) 1996-99, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  wlistctn
 *
 *  Author:  Andrew Hewett
 *
 *  Purpose: Service Class Executive (SCE)
 *
 *  Last Update:      $Author: meichel $
 *  Update Date:      $Date: 1999/04/26 12:59:10 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/apps/sce.cc,v $
 *  CVS/RCS Revision: $Revision: 1.7 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#include "osconfig.h"    /* make sure OS specific configuration is included first */

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
END_EXTERN_C

#include "dicom.h"
#include "wlistctn.h"
#include "wrklstdb.h"
#include "dimse.h"
#include "diutil.h"
#include "sce.h"
#ifndef NODISPLAY
#include "dispuser.h"
#endif

#include "sceecho.h"
#include "scefind.h"


/*
 *
 */

static CONDITION
SCE_dispatch(T_ASC_Association *assoc)
{
    CONDITION cond = DIMSE_NORMAL;
    T_DIMSE_Message msg;
    T_ASC_PresentationContextID presID;
        
    /* Create a database handle for this association */
    char wlistpath[2048];
    sprintf(wlistpath, "%s/%s", baseWorklistDBPath, 
	    assoc->params->DULparams.calledAPTitle);

    WorklistDB dbHandle(wlistpath);
    dbHandle.setFailInvalidQuery(opt_failInvalidQuery>0);
 
    while (cond == DIMSE_NORMAL) {
	
	cond = DIMSE_receiveCommand(assoc, DIMSE_BLOCKING, 0, &presID,
		&msg, NULL);

	/* did peer release, abort, or do we have a valid message ? */
	switch (cond) {
	case DIMSE_NORMAL:
	    /* process command */
	    switch (msg.CommandField) {
	    case DIMSE_C_ECHO_RQ:
		cond = SCE_echoSCP(assoc, &msg.msg.CEchoRQ, presID);
		break;
	    case DIMSE_C_FIND_RQ:
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_FindOperation,
			DSU_StartOperation, DSU_Incoming, 0);
#endif
		cond = SCE_findSCP(assoc, &msg.msg.CFindRQ, presID, 
		    dbHandle);
#ifndef NODISPLAY
		DSU_operationState(DSU_MainAssociation, DSU_FindOperation,
			DSU_StopOperation, DSU_Incoming, 100);
#endif
		break;
	    case DIMSE_C_CANCEL_RQ:
		/* This is a late cancel request, just ignore it */
		if (opt_verbose) {
 		    printf("WARNING: late C-CANCEL-RQ, ignoring\n");
 		}
		break;
	    default:
		/* we cannot handle this kind of message */
		cond = DIMSE_BADCOMMANDTYPE;
		errmsg("Cannot handle command: 0x%x\n", 
			(unsigned)msg.CommandField);
	        /* the condition will be returned, the caller
	         * will abort the assosiation.
	         */
	    }
	    break;
	case DIMSE_PEERREQUESTEDRELEASE:
	case DIMSE_PEERABORTEDASSOCIATION:
	    /* association gone */
	    break;
	default:
	    /* the condition will be returned, the caller
	     * will abort the assosiation.
	     */
	    break;
	}
    }

    return cond;
}

CONDITION
SCE_handleAssociation(T_ASC_Association * assoc)
{
    CONDITION           cond;
    DIC_NODENAME	peerHostName;
    DIC_AE		peerAETitle;
    DIC_AE		myAETitle;

    ASC_getPresentationAddresses(assoc->params, peerHostName, NULL);
    ASC_getAPTitles(assoc->params, peerAETitle, myAETitle, NULL);

#ifndef NODISPLAY
    DSU_startAssociation(peerHostName, peerAETitle, myAETitle,
			 DSU_Acceptor, DSU_MainAssociation);
#endif
    
 /* now do the real work */
    cond = SCE_dispatch(assoc);

 /* clean up on association termination */
    if (cond == DIMSE_PEERREQUESTEDRELEASE) {
	COND_PopCondition(OFFalse);	/* pop only the peer requested
					 * release condition from the stack */
	if (opt_verbose)
	    printf("Association Release\n");
	cond = ASC_acknowledgeRelease(assoc);
	ASC_dropSCPAssociation(assoc);
    } else if (cond == DIMSE_PEERABORTEDASSOCIATION) {
	COND_PopCondition(OFFalse);	/* pop DIMSE abort */
	COND_PopCondition(OFFalse);	/* pop DUL abort */
	if (opt_verbose)
	    printf("Association Aborted\n");
    } else {
	errmsg("DIMSE Failure (aborting association):\n");
	COND_DumpConditions();
    /* some kind of error so abort the association */
	cond = ASC_abortAssociation(assoc);
    }

    cond = ASC_dropAssociation(assoc);
    if (!SUCCESS(cond)) {
	fprintf(stderr, "Cannot Drop Association:\n");
	COND_DumpConditions();
    }
    cond = ASC_destroyAssociation(&assoc);
    if (!SUCCESS(cond)) {
	fprintf(stderr, "Cannot Destroy Association:\n");
	COND_DumpConditions();
    }

#ifndef NODISPLAY
    /* inform display that the association has ended */
    DSU_stopAssociation(DSU_MainAssociation);
#endif

    if (opt_verbose) {
        printf("+++++++++++++++++++++++++++++\n");
    }
    return cond;
}


/*
** CVS Log
** $Log: sce.cc,v $
** Revision 1.7  1999/04/26 12:59:10  meichel
** Ported Worklist CTN to Win32 environment.
**
** Revision 1.6  1998/01/27 10:51:51  meichel
** Removed some unused variables, meaningless const modifiers
**   and unreached statements.
**
** Revision 1.5  1997/08/05 07:38:20  andreas
** Corrected error in DUL finite state machine
** SCPs shall close sockets after the SCU have closed the socket in
** a normal association release. Therfore, an ARTIM timer is described
** in DICOM part 8 that is not implemented correctly in the
** DUL. Since the whole DUL finite state machine is affected, we
** decided to solve the proble outside the fsm. Now it is necessary to call the
** ASC_DropSCPAssociation() after the calling ASC_acknowledgeRelease().
**
** Revision 1.4  1997/07/21 09:08:51  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.3  1996/09/27 08:54:25  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C
**
** Revision 1.2  1996/08/29 16:16:06  hewett
** Corrected copyright header.
**
** Revision 1.1.1.1  1996/04/26 14:16:01  hewett
** Initial Import.
**
*/
